import com.urbancode.anthill3.*
import groovy.xml.StreamingMarkupBuilder
import java.text.SimpleDateFormat

final def workDir = new File('.').canonicalFile
final def out = System.out

private final def AHP_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.S Z");
AHP_DATE_FORMAT.timeZone = TimeZone.getTimeZone("GMT")

private final def TFS_DATE_FORMAT = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss 'Z'")
TFS_DATE_FORMAT.timeZone = TimeZone.getTimeZone("UTC")

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------
final def ahpTool = new AHPTool();

final def props = ahpTool.getStepProperties();

final def serverUrl = props['integration/serverUrl']
final def tftoolExecutable = 'tftool' + props['integration/serverVersion'] + '.exe'
final def buildUri = props['buildUri']
final def repositoryId = props['repositoryId']

def Set changeSetSet = []
try {
    //------------------------------------------------------------------------------
    // Get build information from TFS
    //------------------------------------------------------------------------------

    try {
        new CommandHelper(workDir).runCommand("Get TFS Build info for $buildUri",
                [tftoolExecutable, 'buildinfo', "/server:$serverUrl", "/buildUri:$buildUri"]) { proc ->
            proc.consumeProcessErrorStream(out)
            proc.outputStream.close()
            def int index = 0
            proc.in.eachLine{ line ->
                println line
                index ++
                if (index == 6) line.split(',').each { if (it.trim().length() > 0) changeSetSet << it}
            }
        }
    }
    catch (Exception e) {
        throw new Exception("Error obtaining TFS build information: $e.message")
    }

    //------------------------------------------------------------------------------
    // Now get the changelog from TFS
    //------------------------------------------------------------------------------
    if (changeSetSet.size() > 0) {
        try {
            def changeSetXmlBuilder = new StreamingMarkupBuilder()
            def changeSetXml
            def hasChanges = false
            def changeSetDate

            new CommandHelper(workDir).runCommand("Get TFS Changelog",
                    [tftoolExecutable, 'changeset', "/server:$serverUrl", "/changesetIdList:${changeSetSet.join(',')}"]) { proc ->
                proc.consumeProcessErrorStream(System.out)
                proc.out.close()
                def changeLog = new XmlParser().parse(proc.in)

                changeSetXml = changeSetXmlBuilder.bind() {
                    'change-log' {
                        changeLog.'change-sets'[0].'change-set'.each { changeSet ->
                            changeSetDate = TFS_DATE_FORMAT.parse(changeSet.'change-date'[0].text())
                            def changeArray = []
                            changeSet.'change'.each {change ->
                                changeArray << change
                                hasChanges = true
                            }
                            if (changeArray.size() > 0) {
                                'change-set'() {
                                    id(changeSet.'change-id'[0].text())
                                    user(changeSet.'change-user'[0].text())
                                    date(AHP_DATE_FORMAT.format(changeSetDate))
                                    if (changeSet.'change-comment'[0]?.text().trim().length() > 0) {
                                        comment(changeSet.'change-comment'[0].text().trim())
                                    }
                                    'repository-type'('tfs')
                                    'repository-id'(repositoryId)
                                    'file-set' {
                                        changeArray.each {change ->
                                            file('change-type': change.'change-type'[0].text(), change.'change-item'[0].text())
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            ahpTool.addChangeSets(changeSetXml)
        }
        catch (Exception e) {
            throw new Exception("Error obtaining TFS build information: $e.message")
        }
    }
    else {
        println "Did not find any changesets associated with build $buildUri"
    }
}
catch (Throwable t) {
    t.printStackTrace()
    System.exit 1
}