import com.urbancode.anthill3.*
import groovy.xml.StreamingMarkupBuilder

final def workDir = new File('.').canonicalFile
final def out = System.out
final def err = System.err

def statusMap = [
            'Error': 'success',
            'Failed': 'failure',
            'Timeout': 'failure',
            'Aborted': 'failure',
            'Inconclusive': 'failure',
            'PassedButRunAborted': 'success',
            'NotRunnable': 'error',
            'NotExecute': 'failure',
            'Disconnected': 'error',
            'Warning': 'success',
            'Passed': 'success',
            'Completed': 'success',
            'InProgress': 'error',
            'Pending': 'error'
]
def reportName = 'TFS Unit Test Results'

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------
final def ahpTool = new AHPTool();

final def props = ahpTool.getStepProperties();

final def serverUrl = props['integration/serverUrl']
final def tfsVersion = props['integration/serverVersion']
final def tftoolExecutable = 'tftool' + tfsVersion + '.exe'
final def buildUri = props['buildUri']

def scrubName = { name ->
    return name.replaceAll('@', '_').replaceAll(' ', '_').replaceAll(':', '-')
}

if ('2008'.equals(tfsVersion)) {
    println "Please use the MSTest plugin to publish results from the TestResults directory."
}
else {
    try {
        new CommandHelper(workDir).runCommand("Get TFS Build Test Results",
                [tftoolExecutable, 'buildtestresults', "/server:$serverUrl", "/buildUri:$buildUri"]) { proc ->
            proc.consumeProcessErrorStream(err)
            proc.out.close()
            def testReport = new XmlSlurper(false, false).parse(proc.in)

            def builder = new StreamingMarkupBuilder()
            def xml = builder.bind() {
                'test-report'(name: reportName, type: 'MSTest', 'job-id': System.getenv('AH_JOB_ID')) {
                    testReport.'test-run'.each { testRun ->
                        'test-suite'(name: scrubName(testRun.'@name'.text())) {
                            testRun.'test-case'.each { testCase ->
                                test(name: scrubName(testCase.'@name'.text()),
                                        'class-name': testCase.'@class'.text(),
                                        result: statusMap[testCase.'@result'.text()],
                                        time: testCase.'@duration'.text()) {
                                    testCase.'message'.each { msg ->
                                        message(msg.text())
                                    }
                                }
                            }
                        }
                    }
                }
            }

            new CommandHelper(workDir).runCommand("Uploading to AHP", [ ahpTool.ahptool, 'addTests', '-report', reportName, '-']) { process ->
                process.withWriter { it << xml }
                process.waitForProcessOutput(out, err)
            }
        }

    }
    catch (Exception e) {
        println "Error obtaining TFS build test results: $e.message"
        e.printStackTrace()
        System.exit 1
    }
}
