#!/usr/bin/env groovy
import com.urbancode.anthill3.*

final def workDir = new File('.').canonicalFile
final def out = System.out

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------
final def ahpTool = new AHPTool();

final def props = ahpTool.getStepProperties();

final def serverUrl = props['integration/serverUrl']
final def tftoolExecutable = 'tftool' + props['integration/serverVersion'] + '.exe'
final def buildUri = props['buildUri']

def dropLocation = null

try {
    //------------------------------------------------------------------------------
    // PREPARE WORKING DIRECTORY
    //------------------------------------------------------------------------------
    workDir.mkdirs()
    if (!workDir.exists()) {
        throw new Exception("Could not create working directory $workDir")
    } else if (workDir.isFile()) {
        throw new Exception("Specified working directory is a file!! $workDir")
    }

    //------------------------------------------------------------------------------
    // Get build information from TFS
    //------------------------------------------------------------------------------

    try {
        new CommandHelper(workDir).runCommand("Get TFS Build info for $buildUri",
                [tftoolExecutable, 'buildinfo', "/server:$serverUrl", "/buildUri:$buildUri"]) { proc ->
            proc.consumeProcessErrorStream(out)
            proc.outputStream.close()
            proc.in.eachLine { line ->
                println line
                if (dropLocation == null) dropLocation = line
            }
        }
    }
    catch (Exception e) {
        throw new Exception("Error obtaining TFS build information: $e.message")
    }

    println "Found TFS build drop location: $dropLocation"

    //------------------------------------------------------------------------------
    // Now get the artifacts from TFS
    //------------------------------------------------------------------------------
    new CommandHelper(workDir).runCommand('Transfer Build Artifacts',
            ['xcopy', dropLocation + '\\*.*', '.', '/f', '/e', '/h', '/y', '/z'])
}
catch (Throwable t) {
    t.printStackTrace()
    System.exit 1
}